/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Frequenz messen und ausgeben mit einem ATMega328
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include "i2clcd.h"
#include "input_capture.h"
#include "i2ctstsyscom.h"

// Defines -------------------------------------------------------------------
#define TXTLEN 17
#define TESTSYS // Kommunikation mit dem Testsystem

// Variablen -----------------------------------------------------------------
char     txt[TXTLEN];
uint16_t   f = (uint16_t)0;
uint8_t frac =  (uint8_t)0;
uint32_t   T = (uint32_t)0;
double  freq =   (double)0;

// interne Prototypen --------------------------------------------------------
static void cleartext(void);

// Funktionen ----------------------------------------------------------------
void setup() 
{
  // Ausgabe auf LCD
  i2c_lcd_Setup();
  sprintf(txt,"Frequenzmessung:");
  i2c_lcd_Write(I2CLCD_LINE1,txt);
  cleartext();
  sprintf(txt,"f = %5u.%1u Hz  ",f,frac);
  i2c_lcd_Write(I2CLCD_LINE2,txt);
  cleartext();

#ifdef TESTSYS
  // Ausgabe zum Testsystem
  i2c_tstsyscom_Setup();
  sprintf(txt,"Starte Frequenz messen");
  i2c_tstsyscom_Write(txt);
  cleartext();
  delay(1000);
  sprintf(txt,"F min = 31 Hz");
  i2c_tstsyscom_Write(txt);
  cleartext();
  delay(1000);
  sprintf(txt,"F max = 99 Hz");
  i2c_tstsyscom_Write(txt);
  cleartext();
  delay(1000);
#endif
  
  // Frequenzzaehler
  inpcap_Init();
}

void loop() 
{
  // Messwerte holen
  if ((bool)false == inpcap_Found())
  {
    T = inpcap_GetPeriodendauer();
    freq = 1.0 / ((double)T / (double)1000000.0);
  }
  else
  {
    T    = (uint32_t)0;
    freq = (double)0;
  }
  
  // Ausgabe auf LCD
    f    = (uint16_t)freq;                                                      // Vor komma
    frac = (uint16_t)((freq * (double)10.0) - ((uint16_t)freq * (uint16_t)10)); // eine Stelle nach Komma
  if (((double)31.0 > freq) || ((double)99.0 < freq))
  {
    sprintf(txt, "ausser Bereich! ");
    i2c_lcd_Write(I2CLCD_LINE2,txt);
#ifdef TESTSYS
    // Ausgabe zum Testsystem
    i2c_tstsyscom_Write(txt);
#endif
    cleartext();
  }
  else // Ausgabe nur im Bereich von 31 bis 99 Hz
  {
    sprintf(txt, "f = %5u.%1u Hz  ",f, frac);  
    i2c_lcd_Write(I2CLCD_LINE2,txt);
#ifdef TESTSYS
    // Ausgabe zum Testsystem
    i2c_tstsyscom_Write(txt);
#endif
    cleartext();
    f    = (uint16_t)0;
    frac = (uint16_t)0;
  }
  delay(1000);
}

// Textausgabefeld initialisieren
static void cleartext(void)
{
  int i = 0;

  for(i = 0; i < (TXTLEN - 1); i++)
  {
    txt[i] = EMPTY;
  }
  txt[TXTLEN - 1] = ENDE;
}
